@@ -1,11 +1,10 @@
 from denormalizer import *
-from browser import *
 from star import *
-from mapper import *
+from workspace import *
 
 __all__ = []
 
 __all__ += denormalizer.__all__
-__all__ += browser.__all__
 __all__ += star.__all__
-__all__ += mapper.__all__
+__all__ += workspace.__all__
+
