@@ -35,35 +35,6 @@ __all__ = [
     "QueryContext"
 ]
 
-# Required functionality checklist
-# 
-# * [done] fact
-# * [partial] facts in a cell
-# *     [done] pagination
-# *     [done] ordering
-# * [partial] aggregation
-# *     [done] drill-down
-# *     [done] drill-down pagination
-# *     [done] number of total items in drill-down
-# *     [done] drill-down ordering
-# *     [ ] drill-down limits (such as top-10)
-# *     [ ] remainder
-# * [ ] ratio - aggregate sum(current)/sum(total) 
-# * [ ] derived measures
-# * [partial] dimension values
-# *     [done] pagination
-# *     [done] ordering
-
-# Browsing context:
-#     * engine
-#     * metadata
-#
-#     * locale
-#
-#     * fact name
-#     * dimension table prefix
-#     * schema
-
 class SnowflakeBrowser(AggregationBrowser):
     """docstring for SnowflakeBrowser"""
 
@@ -83,6 +54,14 @@ class SnowflakeBrowser(AggregationBrowser):
         * `options` - passed to the mapper and context (see their respective
           documentation)
 
+        Tuning:
+
+        * `include_summary` - it ``True`` then summary is included in
+          aggregation result. Turned on by default.
+        * `include_cell_count` – if ``True`` then total cell count is included
+          in aggregation result. Turned on by default, might be turned off for
+          performance reasons
+
         Limitations:
 
         * only one locale can be used for browsing at a time
@@ -104,6 +83,8 @@ class SnowflakeBrowser(AggregationBrowser):
             self.connectable = connectable
             self.metadata = metadata or sqlalchemy.MetaData(bind=self.connectable)
 
+        self.include_summary = options.get("include_summary", True)
+        self.include_cell_count = options.get("include_cell_count", True)
         # Mapper is responsible for finding corresponding physical columns to
         # dimension attributes and fact measures. It also provides information
         # about relevant joins to be able to retrieve certain attributes.
@@ -121,12 +102,24 @@ class SnowflakeBrowser(AggregationBrowser):
 
         # QueryContext is creating SQL statements (using SQLAlchemy). It
         # also caches information about tables retrieved from metadata.
-
+        # FIXME: new context is created also when locale changes in set_locale
+        self.options = options
         self.context = QueryContext(self.cube, self.mapper,
-                                      metadata=self.metadata)
+                                      metadata=self.metadata, **self.options)
+
+    def set_locale(self, locale):
+        """Change the browser's locale"""
+        self.logger.debug("changing browser's locale to %s" % locale)
+        self.mapper.set_locale(locale)
+        self.locale = locale
+        # Reset context
+        self.context = QueryContext(self.cube, self.mapper,
+                                      metadata=self.metadata, **self.options)
 
     def fact(self, key_value):
-        """Get a single fact with key `key_value` from cube."""
+        """Get a single fact with key `key_value` from cube.
+
+        Number of SQL queries: 1."""
 
         select = self.context.fact_statement(key_value)
 
@@ -136,7 +129,7 @@ class SnowflakeBrowser(AggregationBrowser):
         cursor = self.connectable.execute(select)
         row = cursor.fetchone()
 
-        labels = [c.name for c in select.columns]
+        labels = self.context.logical_labels(select.columns)
 
         if row:
             # Convert SQLAlchemy object into a dictionary
@@ -149,9 +142,10 @@ class SnowflakeBrowser(AggregationBrowser):
         return record
 
     def facts(self, cell, order=None, page=None, page_size=None):
-        """Return all facts from `cell`, might be ordered and paginated."""
+        """Return all facts from `cell`, might be ordered and paginated.
 
-        # TODO: add ordering (ORDER BY)
+        Number of SQL queries: 1.
+        """
 
         statement = self.context.denormalized_statement()
         cond = self.context.condition_for_cell(cell)
@@ -159,20 +153,21 @@ class SnowflakeBrowser(AggregationBrowser):
         if cond.condition is not None:
             statement = statement.where(cond.condition)
 
-        statement = paginated_statement(statement, page, page_size)
-        statement = ordered_statement(statement, order, context=self.context)
+        statement = self.context.paginated_statement(statement, page, page_size)
+        # FIXME: use level based ordering here. What levels to consider? In
+        # what order?
+        statement = self.context.ordered_statement(statement, order)
 
         if self.debug:
             self.logger.info("facts SQL:\n%s" % statement)
 
         result = self.connectable.execute(statement)
-
-        labels = [c.name for c in statement.columns]
+        labels = self.context.logical_labels(statement.columns)
 
         return ResultIterator(result, labels)
 
-    def values(self, cell, dimension, depth=None, paths=None, hierarchy=None,
-                page=None, page_size=None, order=None, **options):
+    def values(self, cell, dimension, depth=None, hierarchy=None, page=None,
+               page_size=None, order=None, **options):
         """Return values for `dimension` with level depth `depth`. If `depth`
         is ``None``, all levels are returned.
 
@@ -188,7 +183,6 @@ class SnowflakeBrowser(AggregationBrowser):
         elif depth is not None:
             levels = levels[0:depth]
 
-        # TODO: add ordering (ORDER BY)
         # TODO: this might unnecessarily add fact table as well, there might
         #       be cases where we do not want that (hm, might be? really? note
         #       the cell)
@@ -197,14 +191,19 @@ class SnowflakeBrowser(AggregationBrowser):
         for level in levels:
             attributes.extend(level.attributes)
 
-        statement = self.context.denormalized_statement(attributes=attributes,
-                                                        include_fact_key=False)
         cond = self.context.condition_for_cell(cell)
 
+        statement = self.context.denormalized_statement(attributes=attributes,
+                                                        include_fact_key=False,
+                                                        condition_attributes=
+                                                              cond.attributes)
         if cond.condition is not None:
             statement = statement.where(cond.condition)
-        statement = paginated_statement(statement, page, page_size)
-        statement = ordered_statement(statement, order, context=self.context)
+
+        statement = self.context.paginated_statement(statement, page, page_size)
+        order_levels = [(dimension, levels)]
+        statement = self.context.ordered_statement(statement, order,
+                                                        order_levels)
 
         group_by = [self.context.column(attr) for attr in attributes]
         statement = statement.group_by(*group_by)
@@ -213,20 +212,70 @@ class SnowflakeBrowser(AggregationBrowser):
             self.logger.info("dimension values SQL:\n%s" % statement)
 
         result = self.connectable.execute(statement)
-        labels = [c.name for c in statement.columns]
+        labels = self.context.logical_labels(statement.columns)
 
         return ResultIterator(result, labels)
 
+    def path_details(self, dimension, path, hierarchy=None):
+        """Returns details for `path` in `dimension`. Can be used for
+        multi-dimensional "breadcrumbs" in a used interface.
+
+        Number of SQL queries: 1.
+        """
+
+        statement = self.context.detail_statement(dimension, path, hierarchy)
+        labels = self.context.logical_labels(statement.columns)
+
+        if self.debug:
+            self.logger.info("path details SQL:\n%s" % statement)
+
+        cursor = self.connectable.execute(statement)
+        row = cursor.fetchone()
+
+        if row:
+            record = dict(zip(labels, row))
+        else:
+            record = None
+
+        cursor.close()
+
+        return record
+
+
     def aggregate(self, cell=None, measures=None, drilldown=None,
                   attributes=None, page=None, page_size=None, order=None,
-                  **options):
+                  include_summary=None, include_cell_count=None, **options):
         """Return aggregated result.
 
+        Arguments:
+
+        * `cell`: cell to be aggregated
+        * `measures`: list of measures to be considered in aggregation
+        * `drilldown`: list of dimensions or list of tuples: (`dimension`,
+          `hierarchy`, `level`)
+        * `attributes`: list of attributes from drilled-down dimensions to be
+          returned in the result
+
+        Query tuning:
+
+        * `include_cell_count`: if ``True`` (default) then
+          `result.total_cell_count` is
+          computed as well, otherwise it will be ``None``.
+        * `include_summary`: if ``True`` (default) then summary is computed,
+          otherwise it will be ``None``
+
+        Result is paginated by `page_size` and ordered by `order`.
+
         Number of database queries:
 
-        * without drill-down: 1 (summary)
-        * with drill-down: 3 (summary, drilldown, total drill-down record
-          count)
+        * without drill-down: 1 – summary
+        * with drill-down (default): 3 – summary, drilldown, total drill-down
+          record count
+
+        Notes:
+
+        * measures can be only in the fact table
+
         """
 
         if not cell:
@@ -240,29 +289,28 @@ class SnowflakeBrowser(AggregationBrowser):
         if measures:
             measures = [self.cube.measure(measure) for measure in measures]
 
-        result = AggregationResult()
-        result.cell = cell
-        result.measures = measures
+        result = AggregationResult(cell=cell, measures=measures)
 
-        summary_statement = self.context.aggregation_statement(cell=cell,
-                                                     measures=measures,
-                                                     attributes=attributes)
+        if include_summary or \
+                include_summary is None and self.include_summary:
+            summary_statement = self.context.aggregation_statement(cell=cell,
+                                                         measures=measures)
 
-        if self.debug:
-            self.logger.info("aggregation SQL:\n%s" % summary_statement)
+            if self.debug:
+                self.logger.info("aggregation SQL:\n%s" % summary_statement)
 
-        cursor = self.connectable.execute(summary_statement)
-        row = cursor.fetchone()
+            cursor = self.connectable.execute(summary_statement)
+            row = cursor.fetchone()
 
-        if row:
-            # Convert SQLAlchemy object into a dictionary
-            labels = [c.name for c in summary_statement.columns]
-            record = dict(zip(labels, row))
-        else:
-            record = None
+            if row:
+                # Convert SQLAlchemy object into a dictionary
+                labels = self.context.logical_labels(summary_statement.columns)
+                record = dict(zip(labels, row))
+            else:
+                record = None
 
-        cursor.close()
-        result.summary = record
+            cursor.close()
+            result.summary = record
 
         ##
         # Drill-down
@@ -282,23 +330,26 @@ class SnowflakeBrowser(AggregationBrowser):
                                                          attributes=attributes,
                                                          drilldown=drilldown)
 
+            statement = self.context.paginated_statement(statement, page, page_size)
+            statement = self.context.ordered_statement(statement, order,
+                                                                    drilldown)
+
             if self.debug:
                 self.logger.info("aggregation drilldown SQL:\n%s" % statement)
 
-            statement = paginated_statement(statement, page, page_size)
-            statement = ordered_statement(statement, order, context=self.context)
-
             dd_result = self.connectable.execute(statement)
-            labels = [c.name for c in statement.columns]
+            labels = self.context.logical_labels(statement.columns)
 
             result.cells = ResultIterator(dd_result, labels)
 
             # TODO: introduce option to disable this
 
-            count_statement = statement.alias().count()
-            row_count = self.connectable.execute(count_statement).fetchone()
-            total_cell_count = row_count[0]
-            result.total_cell_count = total_cell_count
+            if include_cell_count or \
+                    include_cell_count is None and self.include_cell_count:
+                count_statement = statement.alias().count()
+                row_count = self.connectable.execute(count_statement).fetchone()
+                total_cell_count = row_count[0]
+                result.total_cell_count = total_cell_count
 
         return result
 
@@ -374,8 +425,10 @@ class SnowflakeBrowser(AggregationBrowser):
         physical = self.mapper.map_attributes(attributes)
 
         for attr, ref in zip(attributes, physical):
-            table_ref = (ref.schema, ref.table)
+            alias_ref = (ref.schema, ref.table)
+            table_ref = alias_map.get(alias_ref, alias_ref)
             table = physical_tables.get(table_ref)
+
             if table is None:
                 issues.append(("attribute", "table %s.%s does not exist for attribute %s" % (table_ref[0], table_ref[1], self.mapper.logical(attr)), attr))
             else:
@@ -386,8 +439,8 @@ class SnowflakeBrowser(AggregationBrowser):
 
         return issues
 
-"""A Condition representation. `attributes` - list of attributes involved in the conditions,
-`conditions` - SQL conditions"""
+"""A Condition representation. `attributes` - list of attributes involved in
+the conditions, `conditions` - SQL conditions"""
 Condition = collections.namedtuple("Condition",
                                     ["attributes", "condition"])
 
@@ -395,6 +448,9 @@ Condition = collections.namedtuple("Condition",
 AliasedTable = collections.namedtuple("AliasedTable",
                                     ["schema", "table", "alias"])
 
+# FIXME: Remove/dissolve this class, it is just historical remnant
+# NOTE: the class was meant to contain reusable code by different SQL
+#       backends
 class QueryContext(object):
 
     def __init__(self, cube, mapper, metadata, **options):
@@ -437,16 +493,20 @@ class QueryContext(object):
         #
         self.fact_key = self.cube.key or DEFAULT_KEY_FIELD
         self.fact_name = mapper.fact_name
-        self.fact_table = sqlalchemy.Table(self.fact_name, self.metadata,
+        try:
+            self.fact_table = sqlalchemy.Table(self.fact_name, self.metadata,
                                            autoload=True, schema=self.schema)
+        except sqlalchemy.exc.NoSuchTableError:
+            in_schema = " in schema '%s'" if self.schema else ""
+            msg = "No such fact table '%s'%s." % (self.fact_name, in_schema)
+            raise WorkspaceError(msg)
 
         self.tables = {
                     (self.schema, self.fact_name): self.fact_table
                 }
 
         # Collect all tables and their aliases.
-        # FIXME: this should be cached per-browser/cube instance
-
+        #
         # table_aliases contains mapping between aliased table name and real
         # table name with alias:
         # 
@@ -465,6 +525,14 @@ class QueryContext(object):
             table_alias = (join.detail.schema, join.alias or join.detail.table)
             self.table_aliases[table_alias] = table
 
+        # Mapping where keys are attributes and values are columns
+        self.logical_to_column = {}
+        # Mapping where keys are column labels and values are attributes
+        self.column_to_logical = {}
+
+        self.safe_labels = options.get("safe_labels", False)
+        self.label_counter = 1
+
     def aggregation_statement(self, cell, measures=None,
                               attributes=None, drilldown=None):
         """Return a statement for summarized aggregation. `whereclause` is
@@ -488,7 +556,6 @@ class QueryContext(object):
 
         join_expression = self.join_expression_for_attributes(attributes)
 
-        # TODO: add measures as well
         selection = []
 
         group_by = None
@@ -555,12 +622,14 @@ class QueryContext(object):
         return result
 
     def denormalized_statement(self, attributes=None, expand_locales=False,
-                               include_fact_key=True):
+                               include_fact_key=True, condition_attributes=None):
         """Return a statement (see class description for more information) for
         denormalized view. `whereclause` is same as SQLAlchemy `whereclause`
         for `sqlalchemy.sql.expression.select()`. `attributes` is list of
         logical references to attributes to be selected. If it is ``None``
-        then all attributes are used.
+        then all attributes are used. `condition_attributes` contains list of
+        attributes that are not going to be selected, but are required for
+        WHERE condition.
 
         Set `expand_locales` to ``True`` to expand all localized attributes.
         """
@@ -568,7 +637,13 @@ class QueryContext(object):
         if attributes is None:
             attributes = self.mapper.all_attributes()
 
-        join_expression = self.join_expression_for_attributes(attributes, expand_locales=expand_locales)
+        if condition_attributes:
+            join_attributes = set(attributes) | condition_attributes
+        else:
+            join_attributes = set(attributes)
+
+        join_expression = self.join_expression_for_attributes(join_attributes,
+                                                expand_locales=expand_locales)
 
         columns = self.columns(attributes, expand_locales=expand_locales)
 
@@ -577,8 +652,31 @@ class QueryContext(object):
             columns.insert(0, key_column)
 
         select = sql.expression.select(columns,
-                                    from_obj=join_expression,
-                                    use_labels=True)
+                                       from_obj=join_expression,
+                                       use_labels=True)
+
+        return select
+
+    def detail_statement(self, dimension, path, hierarchy=None):
+        """Returns statement for dimension details. `attributes` should be a
+        list of attributes from one dimension that is one branch
+        (master-detail) of a star/snowflake."""
+
+        dimension = self.cube.dimension(dimension)
+        hierarchy = dimension.hierarchy(hierarchy)
+        attributes = hierarchy.all_attributes()
+
+        expression = self.join_expression_for_attributes(attributes,
+                                                        include_fact=False)
+        columns = self.columns(attributes)
+        select = sql.expression.select(columns,
+                                       from_obj=expression,
+                                       use_labels=True)
+
+        cond = self.condition_for_point(dimension, path, hierarchy)
+        select = select.where(cond.condition)
+
+        print "\n\nSQL:\n%s \n\n" % select
 
         return select
 
@@ -587,36 +685,46 @@ class QueryContext(object):
 
         key_column = self.fact_table.c[self.fact_key]
         condition = key_column == key_value
-        statement = self.denormalized_statement()
 
-        if condition is not None:
-            statement = statement.where(condition)
+        statement = self.denormalized_statement()
+        statement = statement.where(condition)
 
         return statement
 
 
-    def join_expression_for_attributes(self, attributes, expand_locales=False):
+    def join_expression_for_attributes(self, attributes, expand_locales=False,
+                                        include_fact=True):
         """Returns a join expression for `attributes`"""
         physical_references = self.mapper.map_attributes(attributes, expand_locales=expand_locales)
 
         joins = self.mapper.relevant_joins(physical_references)
-        return self.join_expression(joins)
+        return self.join_expression(joins, include_fact)
 
-    def join_expression(self, joins):
+    def join_expression(self, joins, include_fact=True):
         """Create partial expression on a fact table with `joins` that can be
         used as core for a SELECT statement. `join` is a list of joins
         returned from mapper (most probably by `Mapper.relevant_joins()`)
+
+        If `include_fact` is ``True`` (default) then fact table is considered
+        as starting point. If it is ``False`` The first detail table is
+        considered as starting point for joins. This might be useful when
+        getting values of a dimension without cell restrictions.
         """
 
         self.logger.debug("create basic expression with %d joins" % len(joins))
 
-        expression = self.fact_table
+        if include_fact:
+            self.logger.debug("join: starting with fact table")
+            expression = self.fact_table
+        else:
+            self.logger.debug("join: ignoring fact table")
+            expression = None
 
         for join in joins:
-            # self.logger.debug("join detail: %s" % (join.detail, ))
 
             if not join.detail.table or join.detail.table == self.fact_name:
-                raise MappingError("Detail table name should be present and should not be a fact table.")
+                raise MappingError("Detail table name should be present and "
+                                   "should not be a fact table.")
 
             master_table = self.table(join.master.schema, join.master.table)
             detail_table = self.table(join.detail.schema, join.alias or join.detail.table)
@@ -634,9 +742,15 @@ class QueryContext(object):
 
             onclause = master_column == detail_column
 
-            expression = sql.expression.join(expression,
+            if expression is not None:
+                expression = sql.expression.join(expression,
                                                     detail_table,
                                                     onclause=onclause)
+            else:
+                self.logger.debug("join: starting with detail table '%s'" %
+                                                                detail_table)
+                expression = detail_table
+
         return expression
 
     def condition_for_cell(self, cell):
@@ -795,6 +909,94 @@ class QueryContext(object):
 
         return Condition(attributes, condition)
 
+    def paginated_statement(self, statement, page, page_size):
+        """Returns paginated statement if page is provided, otherwise returns
+        the same statement."""
+
+        if page is not None and page_size is not None:
+            return statement.offset(page * page_size).limit(page_size)
+        else:
+            return statement
+
+    def ordered_statement(self, statement, order, dimension_levels=None):
+        """Returns a SQL statement which is ordered according to the `order`. If
+        the statement contains attributes that have natural order specified, then
+        the natural order is used, if not overriden in the `order`.
+
+        `dimension_levels` is list of considered dimension levels in form of
+        tuples (`dimension`, `levels`). For each level it's sort key is used.
+        """
+
+        # Each attribute mentioned in the order should be present in the selection
+        # or as some column from joined table. Here we get the list of already
+        # selected columns and derived aggregates
+
+        selection = collections.OrderedDict()
+
+        # Get logical attributes from column labels (see logical_labels method
+        # description for more information why this step is necessary)
+        logical = self.logical_labels(statement.columns)
+        for column, ref in zip(statement.columns, logical):
+            selection[ref] = column
+
+        # Make sure that the `order` is a list of of tuples (`attribute`,
+        # `order`). If element of the `order` list is a string, then it is
+        # converted to (`string`, ``None``).
+
+        order = order or []
+
+        if dimension_levels:
+            for dim, levels in dimension_levels:
+                dim = self.cube.dimension(dim)
+                for level in levels:
+                    level = dim.level(level)
+                    if level.order:
+                        order.append( (level.order_attribute.ref(), level.order) )
+
+        order_by = collections.OrderedDict()
+
+        for item in order:
+            if isinstance(item, basestring):
+                try:
+                    column = selection[item]
+                except KeyError:
+                    attribute = self.mapper.attribute(item)
+                    column = self.column(attribute)
+
+            else:
+                # item is a two-element tuple where first element is attribute
+                # name and second element is ordering
+                try:
+                    column = selection[item[0]]
+                except KeyError:
+                    attribute = self.mapper.attribute(item[0])
+                    column = self.column(attribute)
+
+                column = order_column(column, item[1])
+
+            if item not in order_by:
+                order_by[item] = column
+
+        # Collect natural order for selected columns
+        for (name, column) in selection.items():
+            try:
+                # Backward mapping: get Attribute instance by name. The column
+                # name used here is already labelled to the logical name
+                attribute = self.mapper.attribute(name)
+            except KeyError:
+                # Since we are already selecting the column, then it should
+                # exist this exception is raised when we are trying to get
+                # Attribute object for an aggregate - we can safely ignore
+                # this.
+
+                # TODO: add natural ordering for measures (may be nice)
+                attribute = None
+
+            if attribute and attribute.order and name not in order_by.keys():
+                order_by[name] = order_column(column, attribute.order)
+
+        return statement.order_by(*order_by.values())
+
     def table(self, schema, table_name):
         """Return a SQLAlchemy Table instance. If table was already accessed,
         then existing table is returned. Otherwise new instance is created.
@@ -832,15 +1034,34 @@ class QueryContext(object):
         to be used. If not specified, then the current browsing/mapping locale
         is used for localizable attributes."""
 
+        logical = self.mapper.logical(attribute, locale)
+        if logical in self.logical_to_column:
+            return self.logical_to_column[logical]
+
         ref = self.mapper.physical(attribute, locale)
         table = self.table(ref.schema, ref.table)
-        column = table.c[ref.column]
+        try:
+            column = table.c[ref.column]
+        except:
+            # FIXME: do not expose this exception to server
+            avail = [str(c) for c in table.columns]
+            raise BrowserError("Unknown column '%s' in table '%s' avail: %s" %
+                                        (ref.column, ref.table, avail))
 
         # Extract part of the date
         if ref.extract:
             column = sql.expression.extract(ref.extract, column)
 
-        column = column.label(self.mapper.logical(attribute, locale))
+        if self.safe_labels:
+            label = "a%d" % self.label_counter
+            self.label_counter += 1
+        else:
+            label = logical
+
+        column = column.label(label)
+
+        self.logical_to_column[logical] = column
+        self.column_to_logical[label] = logical
 
         return column
 
@@ -860,6 +1081,23 @@ class QueryContext(object):
 
         return columns
 
+    def logical_labels(self, columns):
+        """Returns list of logical attribute labels from list of columns
+        or column labels.
+
+        This method and additional internal references were added because some
+        database dialects, such as Exasol, can not handle dots in column
+        names, even when quoted.
+        """
+
+        attributes = []
+
+        for column in columns:
+            attributes.append(self.column_to_logical.get(str(column),
+                                                         str(column)))
+
+        return attributes
+
 class AggregatedCubeBrowser(AggregationBrowser):
     """docstring for SnowflakeBrowser"""
 
@@ -915,77 +1153,9 @@ class AggregatedCubeBrowser(AggregationBrowser):
         self.context = QueryContext(self.cube, self.mapper,
                                       metadata=self.metadata)
 
-def paginated_statement(statement, page, page_size):
-    """Returns paginated statement if page is provided, otherwise returns
-    the same statement."""
-
-    if page is not None and page_size is not None:
-        return statement.offset(page * page_size).limit(page_size)
-    else:
-        return statement
-
-def ordered_statement(statement, order, context):
-    """Returns a SQL statement which is ordered according to the `order`. If
-    the statement contains attributes that have natural order specified, then
-    the natural order is used, if not overriden in the `order`."""
-
-    # Each attribute mentioned in the order should be present in the selection
-    # or as some column from joined table. Here we get the list of already
-    # selected columns and derived aggregates
-
-    selection = collections.OrderedDict()
-    for c in statement.columns:
-        selection[str(c)] = c
-
-    # Make sure that the `order` is a list of of tuples (`attribute`,
-    # `order`). If element of the `order` list is a string, then it is
-    # converted to (`string`, ``None``).
-
-    order = order or []
-    order_by = collections.OrderedDict()
-
-    for item in order:
-        if isinstance(item, basestring):
-            try:
-                attribute = context.mapper.attribute(item)
-                column = context.column(attribute)
-            except KeyError:
-                column = selection[item]
-
-            order_by[item] = column
-        else:
-            # item is a two-element tuple where first element is attribute
-            # name and second element is ordering
-            try:
-                attribute = context.mapper.attribute(item[0])
-                column = context.column(attribute)
-            except KeyError:
-                column = selection[item[0]]
-            order_by[item] = order_column(column, item[1])
-
-    # Collect natural order for selected columns
-
-    # TODO: should we add natural order for columns that are not selected
-    #       but somewhat involved in the process (GROUP BY)?
-
-    for (name, column) in selection.items():
-        try:
-            # Backward mapping: get Attribute instance by name. The column
-            # name used here is already labelled to the logical name
-            attribute = context.mapper.attribute(name)
-        except KeyError:
-            # Since we are already selecting the column, then it should exist
-            # this exception is raised when we are trying to get Attribute
-            # object for an aggregate - we can safely ignore this.
-
-            # TODO: add natural ordering for measures (may be nice)
-            attribute = None
-
-        if attribute and attribute.order and name not in order_by.keys():
-            order_by[name] = order_column(column, attribute.order)
-
-    return statement.order_by(*order_by.values())
-
+        # Map: logical attribute --> 
+        self.attribute_columns = {}
+        self.alias_columns
 
 def order_column(column, order):
     """Orders a `column` according to `order` specified as string."""
