@@ -28,6 +28,7 @@ __all__ = [
     "string_from_cuts",
     "string_from_path",
     "string_from_hierarchy",
+    "string_to_drilldown",
     "path_from_string",
     "cut_from_string",
     "cut_from_dict",
@@ -58,28 +59,10 @@ class AggregationBrowser(object):
 
     def full_cube(self):
         # FIXME: Depreciate this method in favor of Cell(cube)
+        logger = get_logger()
+        logger.warn("browser.full_cube() is depreciated, use Cell(cube)")
         return Cell(self.cube)
 
-    def dimension_object(self, dimension):
-        """Helper function to return proper dimension object as a subclass of
-        Dimension.
-
-        .. Warning::
-
-            Depreciated. Use :meth:`cubes.Cube.dimension`
-
-        Arguments:
-
-        * `dimension` - a dimension object or a string, if it is a string,
-          then dimension object is retrieved from cube
-        """
-        # FIXME: depreciate, use cube.dimension(...) directly
-
-        if type(dimension) == str:
-            return self.cube.dimension(dimension)
-        else:
-            return dimension
-
     def aggregate(self, cell=None, measures=None, drilldown=None, **options):
         """Return aggregate of a cell.
 
@@ -262,9 +245,9 @@ class AggregationBrowser(object):
                     cut["details"] = detail
 
                 result = cell_dict
-
             else:
-                raise ArgumentError("Unknown report query '%s' for '%s'" % (query_type, result_name))
+                raise ArgumentError("Unknown report query '%s' for '%s'" %
+                                                    (query_type, result_name))
 
             report_result[result_name] = result
 
@@ -288,8 +271,6 @@ class AggregationBrowser(object):
             one.
         """
 
-        # TODO: is the name right?
-        # TODO: dictionary or class representation?
         # TODO: how we can add the cell as well?
         if not cell:
             return []
@@ -304,30 +285,29 @@ class AggregationBrowser(object):
         return details
 
     def cut_details(self, cut):
-        """Returns details for a `cut` which should be a `Cut` instance.
+        """Gets details for a `cut` which should be a `Cut` instance.
 
         * `PointCut` - all attributes for each level in the path
         * `SetCut` - list of `PointCut` results, one per path in the set
         * `RangeCut` - `PointCut`-like results for lower range (from) and
           upper range (to)
 
-        Default implemenatation uses `AggregationBrowser.values()` for each
-        path. Backends might customize this method to make it more efficient.
-
         """
 
         dimension = self.cube.dimension(cut.dimension)
 
         if isinstance(cut, PointCut):
-            details = self._path_details(dimension, cut.path)
+            details = self._path_details(dimension, cut.path, cut.hierarchy)
 
         elif isinstance(cut, SetCut):
-            details = [self._path_details(dimension, path) for path in cut.paths]
+            details = [self._path_details(dimension, path, cut.hierarchy) for path in cut.paths]
 
         elif isinstance(cut, RangeCut):
             details = {
-                "from": self._path_details(dimension, cut.from_path),
-                "to": self._path_details(dimension, cut.to_path)
+                "from": self._path_details(dimension, cut.from_path,
+                                            cut.hierarchy),
+                "to": self._path_details(dimension, cut.to_path,
+                                            cut.hierarchy)
             }
 
         else:
@@ -354,17 +334,12 @@ class AggregationBrowser(object):
 
         cut = PointCut(dimension, path)
         cell = Cell(self.cube, cuts=[cut])
-        dim_values = list(self.values(cell, dimension, len(path)))
 
-        if len(dim_values) > 1:
-            raise Exception("There are more than one detail rows for dimension %s path %s" % \
-                                (str(dimension), path) )
+        details = self.path_details(dimension, path, hierarchy)
 
-        if not dim_values:
+        if not details:
             return None
 
-        details = dim_values[0]
-
         if (dimension.is_flat and not dimension.has_details):
             name = dimension.all_attributes()[0].name
             value = details.get(name)
@@ -400,7 +375,7 @@ class Cell(object):
 
         return result;
 
-    def slice(self, cut, dummy=None):
+    def slice(self, cut):
         """Returns new cell by slicing receiving cell with `cut`. Cut with
         same dimension as `cut` will be replaced, if there is no cut with the
         same dimension, then the `cut` will be appended.
@@ -410,7 +385,6 @@ class Cell(object):
         if isinstance(cut, Dimension) or isinstance(cut, basestring):
             raise CubesError("slice() should now be called with a cut (since v0.9.2). To get "
                              "original behaviour of one-dimension point cut, "
-                             "use point_slice(dim, path) instead or better: "
                              "use cell.slice(PointCut(dim,path))")
 
         cuts = self.cuts[:]
@@ -461,7 +435,7 @@ class Cell(object):
             cuts.append(cut)
         return Cell(cube=self.cube, cuts=cuts)
 
-    def drilldown(self, dimension, value):
+    def drilldown(self, dimension, value, hierarchy=None):
         """Create another cell by drilling down `dimension` next level on
         current level's key `value`.
 
@@ -481,13 +455,16 @@ class Cell(object):
         Works only if the cut for dimension is `PointCut`. Otherwise the
         behaviour is undefined.
 
-        Returns: new derived cell object.
+        If `hierarchy` is not specified (by default) then default dimension
+        hierarchy is used.
+
+        Returns new derived cell object.
         """
         dimension = self.cube.dimension(dimension)
         dim_cut = self.cut_for_dimension(dimension)
 
         old_path = dim_cut.path if dim_cut else []
-        new_cut = PointCut(dimension, old_path + [value])
+        new_cut = PointCut(dimension, old_path + [value], hierarchy=hierarchy)
 
         cuts = [cut for cut in self.cuts if cut is not dim_cut]
         cuts.append(new_cut)
@@ -539,11 +516,10 @@ class Cell(object):
         hierarchy. If there is no level to go up (we are at the top level),
         then the cut is removed.
 
+        If no `hierarchy` is specified, then the default dimension's hierarchy
+        is used.
+
         Returns new cell object.
-
-        .. note::
-
-                Only default hierarchy is currently supported.
         """
 
         # FIXME: make this the default roll-up
@@ -567,7 +543,7 @@ class Cell(object):
         # are removing the cut for the dimension.
 
         if rollup_path:
-            new_cut = PointCut(dimension, rollup_path)
+            new_cut = PointCut(dimension, rollup_path, hierarchy=hierarchy)
             cuts.append(new_cut)
 
         return Cell(cube=self.cube, cuts=cuts)
@@ -650,8 +626,6 @@ class Cell(object):
         """Returns a dictionary of dimension names as keys and level depths
         (index of deepest level)."""
 
-        # TODO: should accept hierarchies
-
         levels = {}
 
         for cut in self.cuts:
@@ -1082,10 +1056,10 @@ class AggregationResult(object):
         `measures` and `levels` from the aggregate query.
 
     """
-    def __init__(self):
+    def __init__(self, cell=None, measures=None):
         super(AggregationResult, self).__init__()
-        self.cell = None
-        self.measures = None
+        self.cell = cell
+        self.measures = measures
         self.levels = None
 
         self.summary = {}
@@ -1129,7 +1103,16 @@ class AggregationResult(object):
 
         return d
 
-    def table_rows(self, dimension, depth=None):
+    def has_dimension(self, dimension):
+        """Returns `True` if the result was drilled down by `dimension` (at
+        any level)"""
+
+        if not self.levels:
+            return False
+
+        return str(dimension) in self.levels
+
+    def table_rows(self, dimension, depth=None, hierarchy=None):
         """Returns iterator of drilled-down rows which yields a named tuple with
         named attributes: (key, label, path, record). `depth` is last level of
         interest. If not specified (set to ``None``) then deepest level for
@@ -1157,7 +1140,7 @@ class AggregationResult(object):
 
         # FIXME: use hierarchy from cut (when implemented)
         dimension = self.cell.cube.dimension(dimension)
-        hierarchy = dimension.hierarchy()
+        hierarchy = dimension.hierarchy(hierarchy)
 
         if self.levels:
             # Convert "levels" to a dictionary:
@@ -1258,6 +1241,9 @@ def cross_table(drilldown, onrows, oncolumns, measures=None):
 
     """
 
+    logger = get_logger()
+    logger.warn("cross_table() is depreciated, use cross table formatter: "
+                "create_formatter(\"cross_table\", ...)")
     matrix = {}
     row_hdrs = []
     column_hdrs = []
