@@ -116,6 +116,15 @@ class Mapper(object):
         else:
             self.simplify_dimension_references = True
 
+        if self.schema:
+            schemastr = "'%s'" % self.schema
+        else:
+            schemastr = "(default)"
+
+        self.logger.debug("mapper options: fact:'%s', schema:%s, "
+                          "simplify: %s" % (self.fact_name, schemastr,
+                                            self.simplify_dimension_references))
+
         self._collect_attributes()
 
     def _collect_attributes(self):
@@ -138,6 +147,11 @@ class Mapper(object):
                     raise Exception("No dimension in attr %s" % attr)
                 self.attributes[self.logical(attr)] = attr
 
+    def set_locale(self, locale):
+        """Change the mapper's locale"""
+        self.locale = locale
+        self._collect_attributes()
+
     def all_attributes(self, expand_locales=False):
         """Return a list of all attributes of a cube. If `expand_locales` is
         ``True``, then localized logical reference is returned for each
@@ -262,6 +276,7 @@ class SnowflakeMapper(Mapper):
         * `dimension_prefix` – prefix for dimension tables
         * `dimension_schema` – schema whre dimension tables are stored (if
           different than fact table schema)
+
         `mappings` is a dictionary where keys are logical attribute references
         and values are table column references. The keys are mostly in the
         form:
