@@ -1,9 +1,6 @@
 # -*- coding=utf -*-
 """Logical model."""
 
-# FIXME: Model constructors contain lots of default initializations. This
-# should be moved to some other place or made optional by a flag
-
 import os
 import re
 import urllib2
@@ -14,7 +11,7 @@ try:
 except ImportError:
     from ordereddict import OrderedDict
 
-from cubes.common import IgnoringDictionary, get_logger
+from cubes.common import IgnoringDictionary, get_logger, to_label
 from cubes.errors import *
 
 try:
@@ -41,9 +38,18 @@ __all__ = [
     "aggregate_ref"
 ]
 
-DIMENSION = 1
-MEASURE = 2
-DETAIL = 3
+def _open_url_resource(resource):
+    """Opens `resource` either as a file with `open()`or as URL with
+    `urllib2.urlopen()`. Returns opened handle. """
+
+    parts = urlparse.urlparse(resource)
+    should_close = True
+    if parts.scheme in ('', 'file'):
+        handle = open(resource)
+    else:
+        handle = urllib2.urlopen(resource)
+    return handle
+
 
 def load_model(resource, translations=None):
     """Load logical model from object reference. `resource` can be an URL,
@@ -54,15 +60,17 @@ def load_model(resource, translations=None):
     * JSON file with a dictionary describing model
     * URL with a JSON dictionary
 
+    `translations` is a dictionary where keys are locale names and values are
+    paths to translation dictionaries (json files).
+
     Raises `ModelError` when model description file does not contain a
     dictionary.
     """
 
     handle = None
     if isinstance(resource, basestring):
-        parts = urlparse.urlparse(resource)
+        handle = _open_url_resource(resource)
         should_close = True
-        handle = open(resource) if parts.scheme in ('', 'file') else urllib2.urlopen(resource)
     else:
         handle = resource
         should_close = False
@@ -80,14 +88,16 @@ def load_model(resource, translations=None):
 
     if translations:
         for lang, path in translations.items():
-            handle = urllib2.urlopen(path)
-            trans = json.load(handle)
-            handle.close()
+            handle = _open_url_resource(path)
+            try:
+                trans = json.load(handle)
+            finally:
+                handle.close()
             model._add_translation(lang, trans)
 
     return model
 
-def create_model(model, cubes=None, dimensions=None):
+def create_model(model, cubes=None, dimensions=None, translations=None):
     """Create a model from a model description dictionary in `model`. This is
     designated way of creating the model from a dictionary.
 
@@ -106,12 +116,8 @@ def create_model(model, cubes=None, dimensions=None):
         	"dimensions": [...]
         }
 
-    .. note::
-
-        Current implementation is the same as passing `description` to the
-        `Model` class initialization. In the future all default constructions
-        will be moved here and the initialization methods will require logical
-        model class instances.
+    `translations` is a dictionary where keys are locale names and values are
+    translation dictionaries. See `Model.localize()` for more information.
     """
 
     model_desc = dict(model)
@@ -155,9 +161,15 @@ def create_model(model, cubes=None, dimensions=None):
             raise ModelError("Duplicate cube '%s'" % cube.name)
         model_cubes[cube.name] = cube
 
-    return Model(dimensions=model_dimensions.values(),
+    model = Model(dimensions=model_dimensions.values(),
             cubes=model_cubes.values(), **model_desc)
 
+    if translations:
+        for lang, trans in translations.items():
+            model._add_translation(lang, trans)
+
+    return model
+
 def _fix_dict_list(obj, key_name="name", warning=None):
     """Returns a list of dictionaries instead of dictionary of dictionaries"""
     if isinstance(obj, dict):
@@ -226,9 +238,22 @@ def create_dimension(obj, dimensions=None):
         try:
             template = dimensions[obj["template"]]
         except KeyError:
-            raise NoSuchDimensionError("Could not find template dimension %s" % obj["template"])
+            raise NoSuchDimensionError("Could not find template dimension %s"
+                                                            % obj["template"])
+
         levels = copy.deepcopy(template.levels)
-        hierarchies = template.hierarchies.values()
+
+        # Create copy of template's hierarchies, but reference newly created
+        # copies of level objects
+        hierarchies = []
+        level_dict = dict((level.name, level) for level in levels)
+
+        for hier in template.hierarchies.values():
+            hier_levels = [level_dict[level.name] for level in hier.levels]
+            hier_copy = Hierarchy(hier.name, hier_levels,
+                                  label=hier.label, info=hier.info)
+            hierarchies.append(hier_copy)
+
         default_hierarchy_name = template.default_hierarchy_name
         label = template.label
         description = template.description
@@ -281,7 +306,8 @@ def create_dimension(obj, dimensions=None):
             hierarchy = Hierarchy("default", levels=hierarchy)
 
         hierarchies = [hierarchy]
-    else:
+
+    elif "hierarchies" in obj:
         hierarchies = _fix_dict_list(obj.get("hierarchies"),
                                      warning="Hierarchies in a dimension (%s) should"
                                              "be a list, not a dictionary" % name)
@@ -301,7 +327,7 @@ def create_dimension(obj, dimensions=None):
                      )
 
 def create_level(obj):
-    """Creates a level from `obj` which can be a `Level` instance, string or 
+    """Creates a level from `obj` which can be a `Level` instance, string or
     a dictionary. If it is a string, then the string represents level name and
     the only one attribute of the level. If `obj` is a dictionary, then the
     keys are:
@@ -313,7 +339,7 @@ def create_level(obj):
 
     Defaults:
 
-    * if no attributes are specified, then one is created with the same name 
+    * if no attributes are specified, then one is created with the same name
       as the level name.
 
     """
@@ -466,7 +492,8 @@ def simple_model(cube_name, dimensions, measures):
 
 class Model(object):
     def __init__(self, name=None, cubes=None, dimensions=None, locale=None,
-                 label=None, description=None, info=None, **kwargs):
+                 label=None, description=None, info=None, mappings=None,
+                 **kwargs):
         """
         Logical Model represents analysts point of view on data.
 
@@ -479,6 +506,24 @@ class Model(object):
         * `label` - human readable name - can be used in an application
         * `description` - longer human-readable description of the model
         * `info` - custom information dictionary
+        * `mappings` – model-wide mappings of logical-to-physical attributes
+
+        The `mappings` is a dictiononary of form::
+
+            {
+                "cubes": {
+                    "cube_name" : { ... }
+                },
+                "dimensions": {
+                    "dimension_name" : {
+                        "dimension_attribute": "physical_attribute"
+                    }
+                }
+            }
+
+        The mappings in `cubes` are the same as mappings in the `Cube`
+        definition. The logical name (keys) of dimension mappings is just
+        dimension attribute name without the dimension prefix.
         """
 
         self.name = name
@@ -487,6 +532,8 @@ class Model(object):
         self.locale = locale
         self.info = info
 
+        self.mappings = mappings
+
         self._dimensions = OrderedDict()
         if dimensions:
             for dim in dimensions:
@@ -601,6 +648,9 @@ class Model(object):
         cubes = [cube.to_dict(**options) for cube in self.cubes.values()]
         out.setnoempty("cubes", cubes)
 
+        if options.get("with_mappings"):
+            out.setnoempty("mappings", self.mappings)
+
         return out
 
     def __eq__(self, other):
@@ -675,16 +725,94 @@ class Model(object):
 
         return True
 
+    # def mappings_for_cube(self, cube):
+    #     """Returns consolidated mappings for `cube`."""
+    #     cube = self.cube(cube)
+    #     if cube.name in self._cube_mappings:
+    #         return self._cube_mappings[cube.name]
+
+    #     try:
+    #         mappings = self.mappings["cubes"][cube.name].copy()
+    #     except KeyError:
+    #         mappings = None
+
+    #     if mappings and cube.mappings():
+    #         raise ModelError("Both old-style mapping (in cube definition) "
+    #                 "and new-style mapping (in model) provided for "
+    #                 "cube %s. It is recommended to provide mapping at "
+    #                 "the model level." %  cube.name)
+
+    #     if cube.mappings():
+    #         mappings = cube.mappings().copy()
+    #     else:
+    #         mappings = mappings or {}
+
+    #     try:
+    #         all_dims = self.mappings["dimensions"]
+    #     except KeyError:
+    #         all_dims = {}
+
+    #     raise NotImplementedError("Continue here")
+
+    #     for dim in cube.dimensions():
+    #         if dim.name in all_dims:
+    #             for key, value in all_dims[dim.name]
+
     def _add_translation(self, lang, translation):
         self.translations[lang] = translation
 
     def localize(self, translation):
-        """Return localized version of model"""
+        """Return localized version of the model.
+
+        `translation` might be a string or a dicitonary. If it is a string,
+        then it represents locale name from model's localizations provided on
+        model creation. If it is a dictionary, it should contains full model
+        translation that is going to be applied.
+
+
+        Translation dictionary structure example::
+
+            {
+                "locale": "sk",
+                "cubes": {
+                    "sales": {
+                        "label": "Predaje",
+                        "measures":
+                            {
+                                "amount": "suma",
+                                "discount": {"label": "zľava",
+                                             "description": "uplatnená zľava"}
+                            }
+                    }
+                },
+                "dimensions": {
+                    "date": {
+                        "label": "Dátum"
+                        "attributes": {
+                            "year": "rok",
+                            "month": {"label": "mesiac"}
+                        },
+                        "levels": {
+                            "month": {"label": "mesiac"}
+                        }
+                    }
+                }
+            }
+
+        .. note::
+
+            Whenever master model changes, you should call this method to get
+            actualized localization of the original model.
+        """
 
         model = copy.deepcopy(self)
 
         if type(translation) == str or type(translation) == unicode:
-            translation = self.translations[translation]
+            try:
+                translation = self.translations[translation]
+            except KeyError:
+                raise ModelError("Model has no translation for %s" %
+                                    translation)
 
         if "locale" not in translation:
             raise ValueError("No locale specified in model translation")
@@ -858,6 +986,10 @@ class Cube(object):
         """
 
         if isinstance(obj, basestring):
+            # FIXME: make this default attribute if not provided in the model
+            if obj == "record_count":
+                return Attribute("record_count", label="Count")
+
             lookup = [m for m in self.measures if m.name == obj]
             if lookup:
                 if len(lookup) == 1:
@@ -884,13 +1016,17 @@ class Cube(object):
 
         out = IgnoringDictionary()
         out.setnoempty("name", self.name)
-        out.setnoempty("label", self.label)
         out.setnoempty("info", self.info)
 
-        measures = [m.to_dict() for m in self.measures]
+        if options.get("create_label"):
+            out.setnoempty("label", self.label or to_label(self.name))
+        else:
+            out.setnoempty("label", self.label)
+
+        measures = [m.to_dict(**options) for m in self.measures]
         out.setnoempty("measures", measures)
 
-        details = [a.to_dict() for a in self.details]
+        details = [a.to_dict(**options) for a in self.details]
         out.setnoempty("details", details)
 
         if expand_dimensions:
@@ -1004,6 +1140,9 @@ class Cube(object):
 
         return locale
 
+    def __str__(self):
+        return self.name
+
 class Dimension(object):
     """
     Cube dimension.
@@ -1139,9 +1278,6 @@ class Dimension(object):
         """Get hierarchy object either by name or as `Hierarchy`. If `obj` is
         ``None`` then default hierarchy is returned."""
 
-        # TODO: this should replace default_hierarchy constructed property
-        #       see Issue #46
-
         if obj is None:
             return self._default_hierarchy()
         if isinstance(obj, basestring):
@@ -1167,6 +1303,9 @@ class Dimension(object):
             Depreciated. Use `Dimension.hierarchy()` instead.
 
         """
+        logger = get_logger()
+        logger.warn("Dimension.default_hierarchy is depreciated, use "
+                    "hierarchy() instead")
         return self._default_hierarchy()
 
     def _default_hierarchy(self):
@@ -1228,12 +1367,16 @@ class Dimension(object):
 
         out = IgnoringDictionary()
         out.setnoempty("name", self.name)
-        out.setnoempty("label", self.label)
         out.setnoempty("info", self.info)
         out.setnoempty("default_hierarchy_name", self.default_hierarchy_name)
 
-        out["levels"] = [level.to_dict() for level in self.levels]
-        out["hierarchies"] = [hier.to_dict() for hier in self.hierarchies.values()]
+        if options.get("create_label"):
+            out.setnoempty("label", self.label or to_label(self.name))
+        else:
+            out.setnoempty("label", self.label)
+
+        out["levels"] = [level.to_dict(**options) for level in self.levels]
+        out["hierarchies"] = [hier.to_dict(**options) for hier in self.hierarchies.values()]
 
         # Use only for reading, during initialization these keys are ignored, as they are derived
         # They are provided here for convenience.
@@ -1334,10 +1477,16 @@ class Dimension(object):
     def localize(self, locale):
         localize_common(self, locale)
 
-        level_locales = locale.get("levels")
-        if level_locales:
-            for level in self.levels:
-                level_locale = level_locales.get(level.name)
+        attr_locales = locale.get("attributes", {})
+
+        for attrib in self.all_attributes():
+            if attrib.name in attr_locales:
+                localize_common(attrib, attr_locales[attrib.name])
+
+        level_locales = locale.get("levels") or {}
+        for level in self.levels:
+            level_locale = level_locales.get(level.name)
+            if level_locale:
                 level.localize(level_locale)
 
         hier_locales = locale.get("hierarcies")
@@ -1563,10 +1712,14 @@ class Hierarchy(object):
 
         out = IgnoringDictionary()
         out.setnoempty("name", self.name)
-        out.setnoempty("label", self.label)
         out.setnoempty("levels", [str(l) for l in self.levels])
         out.setnoempty("info", self.info)
 
+        if options.get("create_label"):
+            out.setnoempty("label", self.label or to_label(self.name))
+        else:
+            out.setnoempty("label", self.label)
+
         return out
 
     def localize(self, locale):
@@ -1595,7 +1748,10 @@ class Level(object):
       level).  key will be used as a grouping field for aggregations. Key
       should be unique within level. If not specified, then the first
       attribute is used as key.
-    * `sort_key`: name of attribute that is going to be used for sorting
+    * `order`: ordering of the level. `asc` for ascending, `desc` for
+      descending or might be unspecified.
+    * `order_attribute`: name of attribute that is going to be used for
+      sorting, default is first attribute (usually key)
     * `label_attribute`: name of attribute containing label to be displayed
       (for example: ``customer_name`` for customer level, ``region_name`` for
       region level, ``month_name`` for month level)
@@ -1605,7 +1761,8 @@ class Level(object):
     """
 
     def __init__(self, name, attributes, dimension = None, key=None,
-                 sort_key=None, label_attribute=None, label=None, info=None):
+                 order_attribute=None, order=None, label_attribute=None, label=None,
+                 info=None):
 
         self.name = name
         self.dimension = dimension
@@ -1624,6 +1781,10 @@ class Level(object):
         else:
             raise ModelInconsistencyError("Attribute list should not be empty")
 
+        # Set second attribute to be label attribute if label attribute is not
+        # set. If dimension is flat (only one attribute), then use the only
+        # key attribute as label.
+
         if label_attribute:
             self.label_attribute = self.attribute(label_attribute)
         else:
@@ -1632,7 +1793,15 @@ class Level(object):
             else:
                 self.label_attribute = self.key
 
-        self.sort_key = self.attribute(sort_key)
+        # Set first attribute to be order attribute if order attribute is not
+        # set
+
+        if order_attribute:
+            self.order_attribute = self.attribute(order_attribute)
+        else:
+            self.order_attribute = coalesce_attribute(self.attributes[0], dimension)
+
+        self.order = order
 
     def __eq__(self, other):
         if not other or type(other) != type(self):
@@ -1641,7 +1810,7 @@ class Level(object):
             return False
         elif self.label_attribute != other.label_attribute:
             return False
-        elif self.sort_key != other.sort_key:
+        elif self.order_attribute != other.order_attribute:
              return False
 
         if self.attributes != other.attributes:
@@ -1663,15 +1832,11 @@ class Level(object):
         return str(self.to_dict())
 
     def __deepcopy__(self, memo):
-        if self.sort_key:
-            sort_key = str(self.sort_key)
-        else:
-            sort_key = None
-
         return Level(self.name,
                         attributes=copy.deepcopy(self.attributes,memo),
                         key=self.key.name,
-                        sort_key=sort_key,
+                        order_attribute=self.order_attribute.name,
+                        order=self.order,
                         label_attribute=self.label_attribute.name,
                         info=copy.copy(self.info),
                         label=copy.copy(self.label)
@@ -1682,19 +1847,24 @@ class Level(object):
 
         out = IgnoringDictionary()
         out.setnoempty("name", self.name)
-        out.setnoempty("label", self.label)
         out.setnoempty("info", self.info)
 
+        if options.get("create_label"):
+            out.setnoempty("label", self.label or to_label(self.name))
+        else:
+            out.setnoempty("label", self.label)
+
+
         if full_attribute_names:
             out.setnoempty("key", self.key.ref())
             out.setnoempty("label_attribute", self.label_attribute.ref())
-            if self.sort_key:
-                out.setnoempty("sort_key", self.sort_key.ref())
+            out.setnoempty("order_attribute", self.order_attribute.ref())
         else:
             out.setnoempty("key", self.key.name)
             out.setnoempty("label_attribute", self.label_attribute.name)
-            if self.sort_key:
-                out.setnoempty("sort_key", self.sort_key.name)
+            out.setnoempty("order_attribute", self.order_attribute.name)
+
+        out.setnoempty("order", self.order)
 
         array = []
         for attr in self.attributes:
@@ -1725,6 +1895,11 @@ class Level(object):
 
         attr_locales = locale.get("attributes")
         if attr_locales:
+            logger = get_logger()
+            logger.warn("'attributes' in localization dictionary of levels "
+                        "is depreciated. Use list of `attributes` in "
+                        "localization of dimension")
+
             for attrib in self.attributes:
                 if attrib.name in attr_locales:
                     localize_common(attrib, attr_locales[attrib.name])
@@ -1864,7 +2039,10 @@ class Attribute(object):
                 "ref": self.ref()
             }
         if self.label is not None:
-            d["label"] = self.label
+            if options.get("create_label"):
+                d["label"] = self.label or to_label(self.name)
+            else:
+                d["label"] = self.label
         if self.locales:
             d["locales"] = self.locales
         if self.order is not None:
@@ -1894,7 +2072,7 @@ class Attribute(object):
         if locale:
             if not self.locales:
                 raise ArgumentError("Attribute '%s' is not loalizable "
-                                    "(localization %s requested)" 
+                                    "(localization %s requested)"
                                         % (self.name, locale))
             elif locale not in self.locales:
                 raise ArgumentError("Attribute '%s' has no localization %s "
@@ -1976,17 +2154,28 @@ def split_aggregate_ref(measure):
 
     r = measure.rfind("_")
 
-    if r == -1 or r == len(measure):
-        raise ArgumentError("Invalid aggregate reference '%s'" % measure)
+    if r == -1 or r >= len(measure)-1:
+        if r == -1:
+            meaning = measure + "_sum"
+        else:
+            meaning = measure + "sum"
+
+        raise ArgumentError("Invalid aggregate reference '%s'. "
+                            "Did you mean '%s'?"% (measure, meaning))
 
     return (measure[:r], measure[r+1:])
 
 def localize_common(obj, trans):
-    """Localize common attributes: label and description"""
-    if "label" in trans:
-        obj.label = trans["label"]
-    if "description" in trans:
-        obj.description = trans["description"]
+    """Localize common attributes: label and description. `trans` should be a
+    dictionary or a string. If it is just a string, then only `label` will be
+    localized."""
+    if isinstance(trans, basestring):
+        obj.label = trans
+    else:
+        if "label" in trans:
+            obj.label = trans["label"]
+        if "description" in trans:
+            obj.description = trans["description"]
 
 
 def localize_attributes(attribs, translations):
