@@ -1,3 +1,4 @@
+# -*- coding=utf -*-
 # import logging
 import os.path
 import json
@@ -14,16 +15,16 @@ try:
     from werkzeug.wrappers import Response
     from werkzeug.utils import redirect
     from werkzeug.exceptions import NotFound
-except:
+except ImportError:
     from cubes.common import MissingPackage
     _missing = MissingPackage("werkzeug", "Slicer server")
     Response = redirect = NotFound = _missing
 
 try:
-    from cubes_search.sphinx import SphinxSearcher
-except:
+    import cubes_search
+except ImportError:
     from cubes.common import MissingPackage
-    SphinxSearcher = None
+    cubes_search = None
     # SphinxSearcher = MissingPackage("cubes_search", "Sphinx search ", 
     #                         source = "https://github.com/Stiivi/cubes")
     # Get cubes sphinx search backend from: https://github.com/Stiivi/cubes
@@ -36,15 +37,16 @@ __all__ = (
 )
 
 class ApplicationController(object):
-    def __init__(self, args, app, config):
+    def __init__(self, args, workspace, logger, config):
 
-        self.app = app
+        self.workspace = workspace
         self.args = args
         self.config = config
-        self.logger = app.logger
+        self.logger = logger
 
         self.locale = self.args.get("lang")
-        self.model = self.app.localized_model(self.locale)
+        self.locales = self.workspace.locales
+        self.model = self.workspace.localized_model(self.locale)
 
         if config.has_option("server","json_record_limit"):
             self.json_record_limit = config.get("server","json_record_limit")
@@ -85,12 +87,13 @@ class ApplicationController(object):
         #
 
         self.order = []
-        for order in self.args.getlist("order"):
-            split = order.split(":")
-            if len(split) == 1:
-                self.order.append( (order, None) )
-            else:
-                self.order.append( (split[0], split[1]) )
+        for orders in self.args.getlist("order"):
+            for order in orders.split(","):
+                split = order.split(":")
+                if len(split) == 1:
+                    self.order.append( (order, None) )
+                else:
+                    self.order.append( (split[0], split[1]) )
 
     def index(self):
         handle = open(os.path.join(TEMPLATE_PATH, "index.html"))
@@ -124,9 +127,9 @@ class ApplicationController(object):
     def version(self):
         return self.json_response(self.server_info())
 
-    def locales(self):
+    def get_locales(self):
         """Return list of available model locales"""
-        return self.json_response(self.app.locales)
+        return self.json_response(self.locales)
 
     def json_response(self, obj):
         if self.prettyprint:
@@ -142,7 +145,7 @@ class ApplicationController(object):
 
     def json_request(self):
         content_type = self.request.headers.get('content-type')
-        if content_type == 'application/json':
+        if content_type.split(';')[0] == 'application/json':
             try:
                 result = json.loads(self.request.data)
             except Exception as e:
@@ -156,20 +159,21 @@ class ApplicationController(object):
 class ModelController(ApplicationController):
 
     def show(self):
-        d = self.model.to_dict(with_mappings = False)
+        d = self.model.to_dict(with_mappings=False,create_label=True)
 
         # Add available model locales based on server configuration
-        d["locales"] = self.app.locales;
+        d["locales"] = self.locales;
         return self.json_response(d)
 
     def dimension(self, dim_name):
         dim = self.model.dimension(dim_name)
-        return self.json_response(dim.to_dict())
+        return self.json_response(dim.to_dict(create_label=True))
 
     def _cube_dict(self, cube):
-        d = cube.to_dict(expand_dimensions = True,
-                         with_mappings = False,
-                         full_attribute_names = True
+        d = cube.to_dict(expand_dimensions=True,
+                         with_mappings=False,
+                         full_attribute_names=True,
+                         create_label=True
                          )
 
         return d
@@ -182,12 +186,12 @@ class ModelController(ApplicationController):
         return self.json_response(self._cube_dict(cube))
 
     def list_cubes(self):
-        cubes = [cube.to_dict() for cube in self.model.cubes.values()]
+        cubes = [self._cube_dict(cube) for cube in self.model.cubes.values()]
         return self.json_response(cubes)
 
     def list_cube_dimensions(self, cube_name):
         cube = self.model.cube(cube_name)
-        dimensions = [dim.to_dict() for dim in cube.dimensions]
+        dimensions = [dim.to_dict(create_label=True) for dim in cube.dimensions]
         return self.json_response(dimensions)
 
     def dimension_levels(self, dim_name):
@@ -314,7 +318,7 @@ class CubesController(ApplicationController):
                 cube_name = self.cube.name
 
         self.logger.info("browsing cube '%s' (locale: %s)" % (cube_name, self.locale))
-        self.browser = self.app.workspace.browser(self.cube, self.locale)
+        self.browser = self.workspace.browser(self.cube, self.locale)
 
     def prepare_cell(self):
         cut_strings = self.args.getlist("cut")
@@ -405,9 +409,15 @@ class CubesController(ApplicationController):
             raise NotFoundError(dim_name, "dimension",
                                 message="Dimension '%s' was not found" % dim_name)
 
+        hier_name = self.args.get("hierarchy")
+        hierarchy = dimension.hierarchy(hier_name)
+
         values = self.browser.values(self.cell, dimension, depth=depth,
+                                     hierarchy=hierarchy,
                                      page=self.page, page_size=self.page_size)
 
+        depth = depth or len(hierarchy)
+
         result = {
             "dimension": dimension.name,
             "depth": depth,
@@ -460,13 +470,7 @@ class CubesController(ApplicationController):
         return self.json_response(cell_dict)
 
     def details(self, cube):
-        self.logger.warn("/details is depreciated, use /cell")
-        self.create_browser(cube)
-        self.prepare_cell()
-
-        result = self.browser.cell_details(self.cell)
-
-        return self.json_response(result)
+        raise RequestError("'details' request is depreciated, use 'cell' request")
 
 
 class SearchController(ApplicationController):
@@ -495,26 +499,27 @@ class SearchController(ApplicationController):
                 self.cube = self.model.cubes.values()[0]
                 cube_name = self.cube.name
 
-        self.browser = self.app.workspace.browser(self.cube,
+        self.browser = self.workspace.browser(self.cube,
                                                   locale=self.locale)
 
-        if self.config.has_option("sphinx", "host"):
-            self.sphinx_host = self.config.get("sphinx","host")
+    def create_searcher(self):
+        if self.config.has_section("search"):
+            self.options = dict(self.config.items("search"))
+            self.engine_name = self.config.get("search", "engine")
         else:
-            self.sphinx_host = None
-
-        if self.config.has_option("sphinx", "port"):
-            self.sphinx_port = self.config.getint("sphinx","port")
-        else:
-            self.sphinx_port = None
+            raise CubesError("Search engine not configured.")
+        self.logger.debug("using search engine: %s" % self.engine_name)
+        options = dict(self.options)
+        del options["engine"]
+        self.searcher = cubes_search.create_searcher(self.engine_name,
+                                            browser=self.browser,
+                                            locales=self.locales,
+                                            **options)
 
     def search(self, cube):
         self.create_browser(cube)
+        self.create_searcher()
 
-        if not SphinxSearcher:
-            raise ServerError("Search extension cubes_search is not installed")
-
-        sphinx = SphinxSearcher(self.browser, self.sphinx_host, self.sphinx_port)
 
         dimension = self.args.get("dimension")
         if not dimension:
@@ -527,27 +532,20 @@ class SearchController(ApplicationController):
         if not query:
             raise RequestError("No search query provided")
 
-        zipped = self.args.get("_zip")
+        locale = self.locale
+        if not locale and self.locales:
+            locale = self.locales[0]
 
-        locale_tag = 0
-        if self.locale:
-            for (i, locale) in enumerate(self.app.locales):
-                if locale == self.locale:
-                    locale_tag = i
-                    break
+        self.logger.debug("searching for '%s' in %s, locale %s" % (query,
+            dimension, locale))
 
+        search_result = self.searcher.search(query, dimension, locale=locale)
 
-        search_result = sphinx.search(query, dimension, locale_tag=locale_tag)
-
-        # FIXME: remove "values" - backward compatibility key
         result = {
-            "values": None,
             "matches": search_result.dimension_matches(dimension),
             "dimension": dimension,
             "total_found": search_result.total_found,
-            "locale": self.locale,
-            "_locale_tag": locale_tag,
-            "_browser_locale": self.browser.locale
+            "locale": self.locale
         }
 
         if search_result.error:
